extern crate lru_cache;
extern crate hyper;
extern crate select;

pub mod pirates;
pub mod ninjas;

use select::document::Document;
use select::predicate::Class;

use std::collections::BTreeMap;

#[derive(Debug)]
pub enum Error {
    Http(hyper::error::Error),
    Io(std::io::Error),
    Parse(String)
}

impl From<hyper::error::Error> for Error {
    fn from (err: hyper::error::Error) -> Error {
        Error::Http(err)
    }
}

impl From<std::io::Error> for Error {
    fn from (err: std::io::Error) -> Error {
        Error::Io(err)
    }
}

impl From<std::num::ParseIntError> for Error {
    fn from (err: std::num::ParseIntError) -> Error {
        Error::Parse(format!("{}", err))
    }
}

pub trait Fighter {
    fn power (&self) -> u8;
}

struct Parser {
    fighter_type: &'static str,
    document: Document
}

impl Parser {
    pub fn parse_name (&self) -> Result<String, Error> {
        let name_class = format!("{}-name", self.fighter_type);
        self.document.find(Class(&name_class[..])).first()
                      .ok_or_else(|| Error::Parse(format!("could not find {} element", name_class))).map(|result| result.text())
    }

    pub fn parse_skills (&self) -> Result<Skills, Error> {
        let skill_class = format!("{}-skill", self.fighter_type);
        let score_class = format!("{}-score", self.fighter_type);
        let map: Result<BTreeMap<String, Skill>, Error> = self.document.find(Class(&skill_class[..])).iter().map(|node| {
            let skill = node.text();
            let score = node.parent().ok_or_else(|| Error::Parse(format!("could not get {} parent element", skill_class)))?
                                     .find(Class(&score_class[..])).first()
                                     .ok_or_else(|| Error::Parse(format!("could not get {} element", score_class)))?
                                     .text();

            Result::Ok((skill, Skill { score: score }))
        }).collect();

        Result::Ok(Skills {
            map: map?
        })
    }
}

struct Skills {
    map: BTreeMap<String, Skill>
}

impl Skills {
    pub fn get (&self, key: &str) -> Result<&Skill, Error> {
        self.map.get(key).ok_or_else(|| Error::Parse(format!("could not get {} value", key)))
    }
}

struct Skill {
    score: String
}

impl Skill {
    pub fn as_number (&self) -> Result<u8, std::num::ParseIntError> {
        self.score.parse::<u8>()
    }

    pub fn as_weapons (&self) -> Vec<String> {
        if self.score == "None" {
            return vec![];
        }

        self.score.split(", ").into_iter().map(String::from).collect()
    }
}
