use lru_cache::LruCache;
use hyper::client::Client;
use select::document::Document;

use std::io::Read;

use {Error, Fighter, Parser};

pub struct Ninjas {
    cache: LruCache<String, Ninja>,
    client: Client
}

impl Ninjas {
    pub fn new () -> Ninjas {
        Ninjas::with_cache_size(30)
    }

    pub fn with_cache_size (cache_size: usize) -> Ninjas {
        Ninjas {
            cache: LruCache::new(cache_size),
            client: Client::new()
        }
    }

    pub fn get (&mut self, name: &str) -> Result<&Ninja, Error> {
        if self.cache.contains_key(name) {
            return Result::Ok(self.cache.get_mut(name).unwrap())
        }

        let ninja = self.get_by_name(name)?;
        self.cache.insert(String::from(name), ninja);
        Result::Ok(self.cache.get_mut(name).unwrap())
    }

    fn get_by_name (&self, name: &str) -> Result<Ninja, Error> {
        let mut contents = String::new();
        self.client.get(&format!("http://www.majcher.com/pvn/pvn.cgi?a=n&n1={}", name)).send()?.read_to_string(&mut contents)?;
        Ninja::from_html(&contents[..])
    }
}

#[derive(Debug)]
pub struct Ninja {
    pub name: String,
    pub sneakiness: u8,
    pub pajamas: u8,
    pub pointy_things: u8,
    pub weapons: Vec<String>
}

impl Ninja {
    pub fn from_html (contents: &str) -> Result<Ninja, Error> {
        let parser = Parser {
            fighter_type: "ninja",
            document: Document::from(contents)
        };

        let skills = parser.parse_skills()?;
        Result::Ok(Ninja {
            name: parser.parse_name()?,
            sneakiness: skills.get("Sneakiness:")?.as_number()?,
            pajamas: skills.get("Pajamas:")?.as_number()?,
            pointy_things: skills.get("Pointy Things:")?.as_number()?,
            weapons: skills.get("Weapons:")?.as_weapons()
        })
    }
}

impl Fighter for Ninja {
    fn power (&self) -> u8 {
        self.sneakiness + self.pajamas + self.pointy_things + self.weapons.len() as u8
    }
}
