use lru_cache::LruCache;
use hyper::client::Client;
use select::document::Document;

use std::io::Read;

use {Error, Fighter, Parser};

pub struct Pirates {
    cache: LruCache<String, Pirate>,
    client: Client
}

impl Pirates {
    pub fn new () -> Pirates {
        Pirates::with_cache_size(30)
    }

    pub fn with_cache_size (cache_size: usize) -> Pirates {
        Pirates {
            cache: LruCache::new(cache_size),
            client: Client::new()
        }
    }

    pub fn get (&mut self, name: &str) -> Result<&Pirate, Error> {
        if self.cache.contains_key(name) {
            return Result::Ok(self.cache.get_mut(name).unwrap())
        }

        let pirate = self.get_by_name(name)?;
        self.cache.insert(String::from(name), pirate);
        Result::Ok(self.cache.get_mut(name).unwrap())
    }

    fn get_by_name (&self, name: &str) -> Result<Pirate, Error> {
        let mut contents = String::new();
        self.client.get(&format!("http://www.majcher.com/pvn/pvn.cgi?a=p&n1={}", name)).send()?.read_to_string(&mut contents)?;
        Pirate::from_html(&contents[..])
    }
}

#[derive(Debug)]
pub struct Pirate {
    pub name: String,
    pub swashbuckling: u8,
    pub drunkenness: u8,
    pub booty: u8,
    pub weapons: Vec<String>
}

impl Pirate {
    pub fn from_html (contents: &str) -> Result<Pirate, Error> {
        let parser = Parser {
            fighter_type: "pirate",
            document: Document::from(contents)
        };

        let skills = parser.parse_skills()?;
        Result::Ok(Pirate {
            name: parser.parse_name()?,
            swashbuckling: skills.get("Swashbuckling:")?.as_number()?,
            drunkenness: skills.get("Drunkenness:")?.as_number()?,
            booty: skills.get("Booty:")?.as_number()?,
            weapons: skills.get("Weapons:")?.as_weapons()
        })
    }
}

impl Fighter for Pirate {
    fn power (&self) -> u8 {
        self.swashbuckling + self.drunkenness + self.booty + self.weapons.len() as u8
    }
}
